/*
    Fecha: 17/05/09
    Autor: Adrin Sergio Buratovich
 Objetivo: Funcionalidad bsica para el ordenamiento por
           columnas de los items de un objeto GRID. 
           Obtener un reordenamiento concatenado de 
           todas las columnas de un objeto GRID partiendo
           de la columna seleccionada y prosiguendo por 
           todas las restantes en el rden en que fueron
           definidas.
   Ultima
 Revisin: 17/05/09 - 22:00:00

  NOTA 01: Se incluye la funcin cStrToken() en reemplazo del
           uso de strtoken() ya que sta ltima produce errores
           en ciertos casos.
           EJEMPLO:

           StrToken('|2|3|4',1,'|') DEVUELVE '2'
           cStrToken('|2|3|4',1,'|') DEVUELVE ''

  NOTA 02: Se incluye la funcin sToc() como modo de recuperacin
           de una fecha desde el resultado de la funcin dTos()
           EJEMPLO:

           dTtos(17/05/09) DEVUELVE '20090517' ( C )
           sToc('20090517') DEVUELVE 17/05/09  ( D )

           Esta funcin slo soporta SET DATE BRITISH / FRENCH
           Si el formato es otro devolvera una fecha vaca ya
           que sta se gedera con cTod()

           PENDIENTE: PARAMETRIZACION DEL FORMATO DEVUELTO 
                      VALIDANDO EL ESTADO "SET DATE"

  NOTA 03: GRIDSETORDER() soporta el modo MULTISELECT de la
           definicin del objeto GRID.

           PENDIENTE: VALIDAR Y ADAPTAR EL PROCESO A LA ESTRUCTURA
                      DE ITEMS EN GRID NO MULTISELECT

*/

//--------------------------------------------------------------//

FUNCTION GRIDSETORDER(_nGridCol)

*-------------------
 Local aDataGrid   := {}
 Local nColGrid    := 0
 Local nRowGrid    := 0
*-------------------
 Local cContOrd    := ""
 Local aContOrd    := {}
 Local aValType    := {{},{}}
 Local aGridIndex  := {}
*-------------------
 Local cSeparador  := "|"
*-------------------
 Private uDatoTemp
*-------------------

 If valtype(_nGridCol)<>"N"
  MsgStop("Error de tipo de dato de orden requerido.","GRIDSETORDER()")
  RETURN(.F.)
 EndIf

 // PENDIENTE: FALTA VALIDAR ESTRUCTURA DE ITEMS EN FUNCION DEL TIPO DE 
 //            DEFINICION DEL GRID

 If valtype(this.Item(_nGridCol))<>"A"
  MsgStop("Error de tipo de contenidos a ser procesados.","GRIDSETORDER()")
  RETURN(.F.)
 EndIf

 // Determinacin de parmetros del proceso
 aDataGrid := AClone(this.Item(_nGridCol))
 nColGrid  := Len(aDataGrid)
 nRowGrid  := this.ItemCount
 aContOrd  := Array(nColGrid)

 // Determinacin de conversores de tipo de dato
 For nCol := 1 To nColGrid
  // Se evala el primer item para obtener los tipos de datos de las columnas
  vValType := ValType(this.Item(1)[nCol])
  Do Case
   Case vValType=='C' // No se efecta conversin
    Aadd(aValType[1],'')
    Aadd(aValType[2],'')
   Case vValType=='D'
    Aadd(aValType[1],'dTos(uDatoTemp)')
    Aadd(aValType[2],'sTod(uDatoTemp)')
   Case vValType=='N'
    Aadd(aValType[1],'Str(uDatoTemp)')
    Aadd(aValType[2],'Val(uDatoTemp)')
   Case vValType=='L'
    Aadd(aValType[1],'if(uDatoTemp,".T.",".F.")')
    Aadd(aValType[2],'(uDatoTemp==".T.")')
   OtherWise
    // Tipo de dato no soportado
    RETURN( .F. )
  EndCase
 Next

 // Generacin de contenidos a ordenar
 For nRow := 1 To nRowGrid
  // Se asigna el contenido de la columna requerida
  uDatoTemp := this.Item(nRow)[_nGridCol]
  // Se efecta la conversin de tipo de datos para iniciar la cadena a ser ordenada
  if Empty(aValType[1][_nGridCol])
   cContOrd := aValType[1][_nGridCol] + cSeparador
  else
   cContOrd := &(aValType[1][_nGridCol]) + cSeparador
  endif
  For nCol := 1 To nColGrid
   // Se asigna el contenido de las columnas restantes
   if nCol<>_nGridCol
    uDatoTemp := this.Item(nRow)[nCol]
    // Se efecta la conversin de tipo de datos para actualizar la cadena a ser ordenada
    if Empty(aValType[1][nCol])
     cContOrd  := cContOrd + aValType[1][nCol] + cSeparador
    else
     cContOrd  := cContOrd + &(aValType[1][nCol]) + cSeparador
    endif

   endif
  Next
  Aadd(aGridIndex,cContOrd)
 Next

 // Ordenamiento del contenido
 // PENDIENTE: ASCENDENTE/DESCENDENTE
 aGridIndex := Asort(aGridIndex)

 // Transferencia del contenido ordenado
 For nRow := 1 To nRowGrid

  cContOrd := aGridIndex[nRow]
  // Se transfiere el contenido de la columna requerida obteniendo
  // la posicin inicial del contenido ordenado
  nPosCont := 1
  uDatoTemp := cStrToken(cContOrd,nPosCont,cSeparador)
  // Se efecta la conversin de tipo de datos para generar iniciar el item 
  if Empty(aValType[2][_nGridCol])
   aContOrd[_nGridCol] := aValType[2][_nGridCol]
  else
   aContOrd[_nGridCol] := &(aValType[2][_nGridCol])
  endif
  // Se transfiere el contenido de las columnas restantes obteniendo
  // las posiciones del contenido ordenado, posteriores a la inicial
  // nPosCont
  For nCol := 1 To nColGrid
   // Slo son actualizadas las columnas diferentes a la requerida
   if nCol<>_nGridCol
    nPosCont++
    // Se efecta la conversin de tipo de datos para actualizar el item 
    uDatoTemp := cStrToken(cContOrd,nPosCont,cSeparador)
    // Se efecta la conversin de tipo de datos para generar iniciar item 
    if Empty(aValType[2][nCol])
     aContOrd[nCol] := aValType[2][nCol]
    else
     aContOrd[nCol] := &(aValType[2][nCol])
    endif
   endif
  Next

  // Se transfiere el contenido ordenado al control GRID
  this.Item(nRow) := Aclone(aContOrd)

 Next

 // Actualizacin de interfaz
 this.Refresh()

RETURN(.T.)

//--------------------------------------------------------------//

FUNCTION cStrToken( _cCadena, _nPosDato, _cSeparador )

// REEMPLAZA EL USO DE STRTOKEN YA QUE PRODUCE ERRORES
// CUANDO EXISTEN POSICIONES INICIALES SIN DATOS
//
// EJEMPLO:
//
//  StrToken('|2|3|4',1,'|') DEVUELVE '2'
// cStrToken('|2|3|4',1,'|') DEVUELVE ''

*------------------------
 Local nByte            := 0
 Local nBytes           := Len(_cCadena)
 Local nSeparador       := 0
*------------------------
 Local cByte            := ""
 Local cDato            := ""
*------------------------

 DEFAULT _cSeparador := '|'
 DEFAULT _nPosDato   := 1

 For nByte := 1 To nBytes

  cByte  := SubStr(_cCadena,nByte,1)

  If cByte == _cSeparador
   nSeparador++
   if nSeparador == _nPosDato
     RETURN( cDato )
   else
    cByte := ""
    cDato := ""
   endif
  EndIf

  cDato := cDato + cByte

 Next

 If nSeparador < ( _nPosDato - 1 )
  cDato := ""
 EndIf

RETURN( cDato )

//----------------------------------------------------------------------//

FUNCTION STOD( _cDtosFecha )

 // DEVUELVE UNA FECHA A PARTIR DE UNA CADENA GENERADA POR DTOS()

*---------------------
 Local cFormatoFecha := "BRITISH"
 Local cFecha        := ""
*---------------------

 // PENDIENTE: EVALUAR ESTADO 'SET DATE' PARA GENERACION DE FECHA
 //            EN TODOS LOS FORMATOS   !!!!!!!

 Do Case
  Case cFormatoFecha == "BRITISH" .or. ;
       cFormatoFecha == "FRENCH"

   cFecha  := SubStr(_cDtosFecha,7,2) + '/' + ;
              SubStr(_cDtosFecha,5,2) + '/' + ;
              SubStr(_cDtosFecha,1,4)

 EndCase

RETURN( cTod(cFecha) )

//----------------------------------------------------------------------//